import org.apache.commons.logging.Log
import org.apache.commons.logging.LogFactory

import com.sas.oprisk.framework.server.objects.BusinessObjectList
import com.sas.oprisk.framework.server.persistence.PersistenceSession
import com.sas.oprisk.server.Dimension
import com.sas.oprisk.server.DimensionNode
import com.sas.oprisk.server.DimensionalPoint
import com.sas.oprisk.server.logical.DimensionalArea

public DimensionalArea getCustomLocation(DimensionalArea controlArea, DimensionalArea testDefArea, PersistenceSession psession){
   Log log = LogFactory.getLog("customNewTestLocation.groovy");
   long timer = System.currentTimeMillis();
   
   DimensionalPoint[] controlPts = controlArea.getPoints();

   log.info("Control getPoints: " + (System.currentTimeMillis() - timer) + "ms" + " size: " + controlPts.length);
    
   timer = System.currentTimeMillis();
   
   DimensionalPoint[] testDefPts = testDefArea.getPoints();
   
   log.info("Test Definition getPoints: " + (System.currentTimeMillis() - timer) + "ms" + " size: " + testDefPts.length);
   timer = System.currentTimeMillis();
   
   List<Dimension> dims = new ArrayList<Dimension>();
   dims.add(Dimension.managementOrg);
   
   Map<String, DimensionNode> nodeMap = new HashMap<String, DimensionNode>();
   
   for (DimensionalPoint dp : controlPts){
      BusinessObjectList<DimensionNode> tmpNodes = dp.withoutDimensions(dims, psession).getNodes(psession);
      log.info("Control DP: " + dp.getDimPointId() + " nodes: " + tmpNodes.size());
      for (DimensionNode n: tmpNodes){
         StringBuffer nodeKey = new StringBuffer(n.getDimension().getClassObject().getPointerTag()).append("_").append(n.getKey());
         nodeMap.put(nodeKey.toString(), n);
      }
   }
   log.info("Control getNodes: " + (System.currentTimeMillis() - timer) + "ms");
   timer = System.currentTimeMillis();
   
   for (DimensionalPoint dp : testDefPts){
      BusinessObjectList<DimensionNode> tmpNodes = dp.withDimensions(dims, psession).getNodes(psession);
      log.info("TestDef DP: " + dp.getDimPointId() + " nodes: " + tmpNodes.size());
      for (DimensionNode n: tmpNodes){
         StringBuffer nodeKey = new StringBuffer(n.getDimension().getClassObject().getPointerTag()).append("_").append(n.getKey());
         nodeMap.put(nodeKey.toString(), n);
      }
   }
   log.info("Test Definition getNodes: " + (System.currentTimeMillis() - timer) + "ms");
   timer = System.currentTimeMillis();
   
   log.info("Nodes collected: " + nodeMap.size());

   BusinessObjectList<DimensionNode> nodes = new BusinessObjectList<DimensionNode>();
   for (Long key : nodeMap.keySet()){
      DimensionNode n = nodeMap.get(key);
      nodes.add(n);
      if (log.infoEnabled){
         log.info("Node: " + n.toString(psession.getLocale(), psession));
      }
   }
   
   DimensionalArea area = DimensionalArea.object.create(nodes, psession);
   log.info("Create area: " + (System.currentTimeMillis() - timer) + "ms");
   
   return area;
}