// ==============================================
//    Table Component Object
// ==============================================

/**
 * Constructor
 */
function TableComponent(
    title,
    doneButtonVisible,
    Cols, 
    rightCols,
    actions,
    display, 
    entryName,
    exportXlsUrl,
    wizCursor,
    required,
    valid,
    sortable,
    disableTableActions,
    fieldName)
{
   /**
    * String literals to be localized
    */
   var RB = new ResourceBundle();
   this.I18N_LOADING=RB.getString("TC.tableComponent.loading.txt");
   this.I18N_NO_RECORDS=RB.getString("TC.tableComponent.noRecords.txt");
   this.I18N_SELECT=RB.getString("TC.tableComponent.select.txt");
   this.I18N_SORT=RB.getString("TC.tableComponent.sort.txt");
   this.I18N_ROW_NUMBER_COLHDG=" ";
   this.I18N_BUTTON_FIRST=RB.getString("TC.tableComponent.button.first.txt");
   this.I18N_BUTTON_PREV=RB.getString("TC.tableComponent.button.prev.txt");
   this.I18N_BUTTON_NEXT=RB.getString("TC.tableComponent.button.next.txt");
   this.I18N_BUTTON_LAST=RB.getString("TC.tableComponent.button.last.txt");
   this.I18N_BUTTON_DONE=RB.getString("TC.tableComponent.button.done.txt");
   this.I18N_BUTTON_CUSTOMIZE=RB.getString("TC.tableComponent.button.customize.txt");
   this.I18N_BUTTON_QUERY=RB.getString("TC.tableComponent.button.query.txt");
   this.I18N_BUTTON_EXPORT=RB.getString("TC.tableComponent.button.export.txt");
   this.I18N_ACTION_MENU_HINT=RB.getString("TC.tableComponent.button.actions.txt");
   this.I18N_BUTTON_CUSTOMIZE_INFO=RB.getString("TC.tableComponent.button.customization.info.txt");
   this.I18N_ENRICH_OPTIONS_TITLE=RB.getString("TC.tableComponent.enrich.options.title.txt");
   this.FILTER_ALL_OPTION=RB.getString("TC.filter.all.txt");
   
   this.RB = RB;

   /**
    * Options
    */
   this.ENABLE_ROW_NUMBER_COLUMN = true;
   this.ENABLE_DEBUG = false;
  // this.MAX_DOWNLOADABLE_ROWS = 5000; // Should be configurable... This property is configured from SMC.
   this.MAX_BUFFER_SIZE =  2000;
   
   this.Cols = Cols;
   this.nLeftCols = Cols ? Cols.length : 0;
   this.rightCols = rightCols;
   this.nRightCols = rightCols ? rightCols.length : 0;
   this.actions = actions;
   this.display = display;
   this.doneButtonVisible = doneButtonVisible;
   this.sortable = (sortable !== false) && !isPrintPage();
   this.disableTableActions = disableTableActions;

   this.entryName = (entryName) ? entryName : "";
   this.fieldName = fieldName;
   
   // Create a new table parameters object
   // and make it an instance variable of this object

   this.tableParameters = new TableParameters(document);
   this.tableParameters.setEntryName(this.entryName);
   
   this.exportXlsUrl = exportXlsUrl;
   this.wizCursor = wizCursor;
   this.required = required;
   this.valid = valid;
   
   this.index = TableComponent.TABLE_COMPONENT_LIST.length;
   TableComponent.TABLE_COMPONENT_LIST.push(this);
   
   if (this.entryName)
   {
      TableComponent.TABLE_COMPONENT_BY_ENTRY_NAME[this.entryName] = this;
   }
   else
   {
      TableComponent.DEFAULT_TABLE_COMPONENT = this;
   }
   
   this.title = title;
    
   // Override the title, if specified.
   if (!title && (typeof tcTitle !== "undefined"))
   {
      this.title = tcTitle;
   }
   
   this.queryFilter = new QueryFilter(this.tableParameters);
   this.render();
}

TableComponent.TABLE_COMPONENT_BY_ENTRY_NAME = {};

TableComponent.TABLE_COMPONENT_LIST = [];

TableComponent.DEFAULT_TABLE_COMPONENT = null;

TableComponent.getLastTableComponent = function() {
   return TableComponent.TABLE_COMPONENT_LIST[TableComponent.TABLE_COMPONENT_LIST.length-1];
};

TableComponent.getTableEntryId = function(entryName, fieldName) {
   if (fieldName == undefined) {
      return entryName;
   } else {
      // TC entry names end in "_" (why?)
      if (entryName.substr(entryName.length-1) != "_") {
         entryName += "_";
      }
      return entryName + fieldName.replace(/\./g, "_") + "_";
   }
}

function getTableComponent(entryName, fieldName) {
  return TableComponent.getTableComponent(entryName, fieldName);
}

TableComponent.getTableComponent = function(entryName, fieldName) {
   if (entryName) {
      // TC entry names end in "_" (why?)
      if (entryName.substr(entryName.length-1) != "_") {
         entryName += "_";
      }
      
      var specificEntryId = TableComponent.getTableEntryId(entryName, fieldName);
      // first see if there is a component under the more specific entry name
      var tc = TableComponent.TABLE_COMPONENT_BY_ENTRY_NAME[specificEntryId];
      // if not found, look under the generic entryName, too 
      if ((tc === undefined) && (specificEntryId != entryName)) {
         tc = TableComponent.TABLE_COMPONENT_BY_ENTRY_NAME[entryName]
      }
      return tc;
   } else {
      return TableComponent.DEFAULT_TABLE_COMPONENT;
   }   
}

// ==============================================
//    Instance methods
// ==============================================

/**
 * Renders the entire table component.
 */
TableComponent.prototype.render = function() {
   var container = this.tableParameters.getTableContainer();
   this.rowCount = null;   
   
   var nCols = this.tableParameters.getFieldCount() 
      + this.nLeftCols + this.nRightCols
      + (this.ENABLE_ROW_NUMBER_COLUMN ? 1 : 0);

   // Create an instance of the table and table body
   // elements.  Make the table element an instance
   // variable
   this.table = this.createTableElement();
   
   var thead = this.table.firstChild;

   // optionally create the title   
   var hasTitle = new Boolean(this.title);
   if (hasTitle || this.actions) {
      
      var tr = document.createElement("tr");
      var th = document.createElement("th");
      th.colSpan = nCols;
      th.style.borderRightWidth = 0;
      th.style.margin = 0;
      th.style.padding = 0;
      
      this.createTitle(this.title, this.actions, th);
      
      tr.appendChild(th);
      thead.appendChild(tr);
   }
   
   // Set the visibility of the entire table 
   if (this.display == false) {
      container.style.display = "none";
   }

   // Create and append the heading row
   thead.appendChild(this.createHeadingRow());

   var tbody = thead.nextSibling;

   // Read the cell data from the hidden fields on 
   // the page and create cells in the table

   var nRows = this.getRowCount();
   for (var row = 0; row < nRows; row++) {
      tbody.appendChild(this.createDetailRow(row));
   }
   if (nRows == 0) {
      var tr = document.createElement("tr");
      tr.className = "empty";
      var td = document.createElement("td");
      
      if (this.tableParameters.getDeferred())
         td.appendChild(document.createTextNode(this.I18N_LOADING));
      else
         td.appendChild(document.createTextNode(this.I18N_NO_RECORDS));
      
      td.colSpan = nCols;
      td.style.borderRightWidth = 0;
      tr.appendChild(td);
      tbody.appendChild(tr);
   }
   
   // Create and add the navigation row at the bottom of the table
      tbody.appendChild(this.createButtonRow(hasTitle));
   
   // add the table to the page body
   container.appendChild(this.table);
   
   // Create and add the "Done" button
   if (this.doneButtonVisible != false) {
       container.appendChild(this.createDialogButtons());
   }
   
   if (this.ENABLE_DEBUG) {
      var debugURL = "javascript:"
         + "var win = window.open('', 'viewsource',"
         +    "'width=600,height=400,resizable,scrollbars');"
         + "win.document.open('text/plain');"
         + "win.document.write(win.opener.document.documentElement.innerHTML);"
         + "win.document.close();"
         ;
      var a = document.createElement("a");
      a.href = debugURL;
      a.style.textDecoration = "none";
      a.style.color = "red";
      a.style.fontSize = "8pt";
      a.style.fontWeight = "bold";
      a.appendChild(document.createTextNode("<DEBUG: View Source>"));
      container.appendChild(a);        
   }
}

/**
 * Refreshes the entire table component.
 */
TableComponent.prototype.refresh = function() {
   this.tableParameters.getTableContainer().innerHTML = "";  // clear the old content
   this.tableParameters = new TableParameters(document);  // document has been refreshed, so create a new TP object
   this.tableParameters.setEntryName(this.entryName);
   this.actions = this.tableParameters.getActionsHTML();
   this.queryFilter = new QueryFilter(this.tableParameters);
   this.render(); // re-render
}

/**
 * Creates the <table> element and adds a <tbody> to it
 */
TableComponent.prototype.createTableElement = function() {

   var table, thead, tbody;

   table = document.createElement("table");
   table.border = 0;
   table.width = "100%";
   
   if (this.title)
   {
      table.summary = this.title;
   }
   
   //table.cellPadding = 4;
   table.cellSpacing = 0;
   table.className = "mainTable";
   thead = document.createElement("thead");
   table.appendChild(thead);
   tbody = document.createElement("tbody");
   table.appendChild(tbody);
   return table;
}

/**
 * Creates the <tr> element containing the headings
 */
TableComponent.prototype.createHeadingRow = function() {
   var tableParameters = this.tableParameters;
   var tr, th;

   tr = document.createElement("tr");

   if (this.ENABLE_ROW_NUMBER_COLUMN) {
      th = document.createElement("th");
      th.className = "tableColumnHeader";
      th.appendChild(document.createTextNode(this.I18N_ROW_NUMBER_COLHDG));
      tr.appendChild(th);
   }
   
   if (this.nLeftCols > 0) {
      this.addFixedColumnHeaderCells(this.Cols, tr);
   }

   var orderByList = tableParameters.getOrderBy();
   var orderByName, orderByDirection;
   if (orderByList != null) {
      orderByName = orderByList[0];
      orderByDirection = orderByList[1];
   }
   
   var nFields = tableParameters.getFieldCount();
   for (var i = 0; i < nFields; i++) {
      th = document.createElement("th");
      th.className = "tableColumnHeader";
      
      if (!this.nRightCols && (i == (nFields - 1)))
      {
         th.style.borderRightWidth = 0;
      }
      
      var name = tableParameters.getFieldName(i);
      
      var textContainerElement;
      
      if (this.sortable && tableParameters.isFieldSortable(name)) {
         textContainerElement = document.createElement("a");
         th.appendChild(textContainerElement);
         
         textContainerElement.href =
               "javascript:getTableComponent('" + this.entryName +
               "').doOrderBy('" + name + "')";
         textContainerElement.title = this.I18N_SORT;
      }
      else {
         textContainerElement = th;
      }
      
      var labelText = unescape(tableParameters.getFormattedLabelForName(name));
      textContainerElement.innerHTML = unescape(tableParameters.getFormattedLabelForName(name));
      th.abbr = unescape(tableParameters.getLabelForName(name));

      if (orderByName == name) {
         textContainerElement.appendChild(this.createImage(
            (orderByDirection == 0 ? "images/thead_SortAscending.gif" : "images/thead_SortDescending.gif")));
      }
      
      tr.appendChild(th);
   }
   
   if (this.nRightCols > 0) {
      this.addFixedColumnHeaderCells(this.rightCols, tr);
   }
   
   return tr;
}

/**
 * Creates a detail row
 */
TableComponent.prototype.createDetailRow = function(row) {

   // Get access to the table parameters object

   var tableParameters = this.tableParameters;

   // Create a table row and assign its CSS class name
   // based on whether it's an odd or even row
   // (so that alternating rows can be different colors)
   // The display table treats its first row as odd, so we'll
   // do the same here.

   var tr = document.createElement("tr");
   tr.className = (row % 2 == 0) ? "dataRow2_tableRow" : "dataRow1_tableRow";

   var start = tableParameters.getStart();
   var rowID = start + row;

   // Make an entry that shows the row number

   if (this.ENABLE_ROW_NUMBER_COLUMN) {
      td = document.createElement("td");
      td.appendChild(document.createTextNode(rowID + 1));
      tr.appendChild(td);
   }
   
   var handle = tableParameters.getHandle(row);
   if (this.Cols !== undefined) {
      this.addFixedColumnCells(this.Cols, tr, handle, row);
   }

   // Loop through all displayed columns, creating cells
   // for each

   var nFields = tableParameters.getFieldCount();
   for (var col = 0; col < nFields; col++) {
      td = this.createCell(row, col);
      td.vAlign = "top";
      
      if (!this.nRightCols && (col == (nFields - 1)))
      {
         td.style.borderRightWidth = 0;
      }

      // Add the newly constructed cell to the row
      tr.appendChild(td);
   }

   // Done
   
   if (this.rightCols) {
      this.addFixedColumnCells(this.rightCols, tr, handle, row);
   }

   return tr;
}

/**
 * Creates the fixed column cells and adds them to the specified row
 */
TableComponent.prototype.addFixedColumnCells = function(fixedCols, tr, handle, row) {
   for (var i = 0; i < fixedCols.length; i++) {
      var fc = fixedCols[i];
      
      var colHTML = unescape(this.tableParameters.getFixedColumnCellValue(row, fc.index));
      colHTML = colHTML.replace(new RegExp("&handle;", "g"), handle);
      if (colHTML == "") colHTML = "&nbsp;"
      
      var td = document.createElement("td");
      td.innerHTML = colHTML;
      
      if ((fixedCols == this.rightCols) && (i == (fixedCols.length - 1)))
      {
         td.style.borderRightWidth = 0;
      }
      
      if (fc.width && fc.width != null) {
         td.width = fc.width;
      }
      if (fc.cssClass && fc.cssClass != null) {
         td.className = fc.cssClass;
      } 
      else if (fc.style && fc.style != null)
      {
         td.style.cssText = fc.style;
      }
      else
      {
         td.align = "center";
      }
      if (fc.valign && fc.valign != null) {
         td.vAlign = fc.valign;
      }
      
      tr.appendChild(td);
   }
}

/**
 * Creates the fixed column header cells and adds them to the specified row
 */
TableComponent.prototype.addFixedColumnHeaderCells = function(fixedCols, tr) {
   for (var i = 0; i < fixedCols.length; i++) {
      var fc = fixedCols[i];
      var title = this.tableParameters.getFixedColumnLabel(fc.index);
      if (!title || (title == "")) {
         title = "&nbsp;";
      } else {
         title = unescape(title);
      }
      
      var th = document.createElement("th");
      th.innerHTML = title;
      th.abbr = title;
      
      if ((fixedCols == this.rightCols) && (i == (fixedCols.length - 1)))
      {
         th.style.borderRightWidth = 0;
      }
      
      if (fc.width && fc.width != null) {
         th.width = fc.width;
      }
      
        th.className = th.className + "tableColumnHeader"
      
      tr.appendChild(th);
   }
}

/**
 * Creates the URL that will be invoked when the edit button is clicked
 */
TableComponent.prototype.getSelectActionURL = function(row) {
   var tableParameters = this.tableParameters;
   var buffer = tableParameters.getSelectActionName();
   if (buffer == "")
      return null;
   //Check if selectAction should be disabled, i;e no hyperlink for this row
   var enableSelectAction = tableParameters.getSelectActionFlag(row); 
   if ( enableSelectAction != null && enableSelectAction == "0" ) {
      return null;
   }

   var handle = tableParameters.getHandle(row);

   if ( buffer.indexOf("javascript:") == 0) {
      var unescHandle = unescape(handle);
      
      // This catch in case someone put '(' in the selectAction 
      if ( buffer.indexOf("(") == -1 ) {
          buffer += "(";
      }
      
      // This catch in Attachment, their handle is more than just handle, 
      // but also include other things like URL, so it would break 
      // the viewAttachment function if we wrap the 3 parameters into 
      // one string under encodeURI
      if (unescHandle.indexOf(",") == -1){
         buffer += "encodeURI(";
      }
      
      buffer += "\"" + unescHandle + "\")";
      
      if (unescHandle.indexOf(",") == -1){
         buffer += ")";
      }
   } else {
       var sep = "?";
       if ( buffer.indexOf("?",0) != -1 ) {
          sep = "&";
       }
       buffer += sep + "handle=" + unescape(handle);
   }
   return buffer;
}

/**
 * Creates a single cell
 */
TableComponent.prototype.createCell = function(row, col) {
   var tableParameters = this.tableParameters;

   // Create the <td> and set its alignment according
   // to its data type

   var td = document.createElement("td");
   var align = tableParameters.getFieldAlignment(col);
   if (align)
   {
      td.align = align;
   }
   
   if (!tableParameters.canFieldWrap(col))
      td.style.whiteSpace = "nowrap";

   // Create the <a> that will contain the cell value
   // and set its URL

   var value = tableParameters.getCellValue(row, col);
   value = unescape(value);
   if (value == "")
      value = "\u00A0";

   var url = this.getSelectActionURL(row); 
   if (url != null) {
      var a = document.createElement("a");
      if (url.indexOf("javascript:") == 0) {
          a.href = url;
      } else {
          a.href = "javascript:doTableSelectAction(encodeURI('" + url + "'))";
      }
      a.title = this.I18N_SELECT;
      
      a.innerHTML = value;
      td.appendChild(a);
   }
   else {
      td.innerHTML = value;
   }

   return td;
}

/**
 * Creates the row of buttons along the bottom
 */
TableComponent.prototype.createButtonRow = function(hasTitle) {
   var tableParameters = this.tableParameters;
   var tr = document.createElement("tr");
   tr.className = "tableNav";

   // Create one cell that spans the entire row.

   var td = document.createElement("td");
   var colSpan = tableParameters.getFieldCount() + this.nLeftCols + this.nRightCols;
   if (this.ENABLE_ROW_NUMBER_COLUMN) {
      colSpan++;
   }
   td.colSpan = colSpan;
   td.style.borderRightWidth = 0;
   td.nowrap = true;
   tr.appendChild(td);
   
   if (!hasTitle && !this.disableTableActions) {
       td.appendChild(this.createActionMenuLink());
   }

   td.appendChild(document.createTextNode("    "));
   
   var hasPrevPage = tableParameters.getStart() > 0;
   
   if (0 != tableParameters.getTotalRows()) {
        // first page link
        var firstButton = document.createElement("a");
        if (hasPrevPage) {
           firstButton.href="javascript:getTableComponent('" + this.entryName + "').doFirst()";
           firstButton.appendChild(this.createImage("images/arrow-up-end-default.gif", this.I18N_BUTTON_FIRST));
        }
        else {
           firstButton.appendChild(this.createImage("images/arrow-up-end-inactive.gif", this.I18N_BUTTON_FIRST));
        }
        td.appendChild(firstButton);
        
        td.appendChild(document.createTextNode(" "));
   }
   
    // previous page link
    var prevButton = document.createElement("a");
    if (hasPrevPage) {
       prevButton.href="javascript:getTableComponent('" + this.entryName + "').doPrev()";
       prevButton.appendChild(this.createImage("images/arrow-up-default.gif", this.I18N_BUTTON_PREV));
    }
    else {
       prevButton.appendChild(this.createImage("images/arrow-up-inactive.gif", this.I18N_BUTTON_PREV));
    }
    td.appendChild(prevButton);
   
    // item X of N message
    if (0 != tableParameters.getTotalRows()) {
       td.appendChild(document.createTextNode("  "));
       
        td.appendChild(
             document.createTextNode(
                this.RB.messageFormat(
                   "TC.tableComponent.items.fmt.txt",
                   tableParameters.getStart() + 1,
                   Math.min((tableParameters.getStart() + tableParameters.getCount()), tableParameters.getTotalRows()),
                   tableParameters.getTotalRows())));
   }
       
   td.appendChild(document.createTextNode("  "));
   
   var hasNextPage = (tableParameters.getMoreRecords() == 1);
   
    // next page link
    var nextButton = document.createElement("a");
    if (hasNextPage) {
       nextButton.href="javascript:getTableComponent('" + this.entryName + "').doNext()";
       nextButton.appendChild(this.createImage("images/arrow-down-default.gif", this.I18N_BUTTON_NEXT));
    }
    else {
       nextButton.appendChild(this.createImage("images/arrow-down-inactive.gif", this.I18N_BUTTON_NEXT));
    }
    td.appendChild(nextButton);
   
    if (0 != tableParameters.getTotalRows()) {
       td.appendChild(document.createTextNode(" "));
       
        // last page link
        var lastButton = document.createElement("a");
        if (hasNextPage) {
           lastButton.href="javascript:getTableComponent('" + this.entryName + "').doLast()";
           lastButton.appendChild(this.createImage("images/arrow-down-end-default.gif", this.I18N_BUTTON_LAST));
        }
        else {
           lastButton.appendChild(this.createImage("images/arrow-down-end-inactive.gif", this.I18N_BUTTON_LAST));
        }
        td.appendChild(lastButton);
    }
   
   return tr;
}

/**
 * Creates a link icon
 */
TableComponent.prototype.createLinkIcon = function(imgSrc, onclick, tip, entryName) {
   var img = this.createImage(imgSrc);
   var result = img;
   if (onclick) {
       var a = document.createElement("a");
       a.style.cursor='hand';
       a.onclick = onclick;
	   a.href="javascript:void(0)";
       a.appendChild(img);
       result = a;
   }
   
   result.id = "showTableActionsLink";
   if (entryName) {
      result.id = "-" + entryName;
   }
   
   if (tip) {
       result.title = tip;
       img.alt = tip;
   }
   return result;
}

/**
 * Creates an image icon
 */
TableComponent.prototype.createImage = function(imgSrc, tip) {
   var img = document.createElement("img");
   img.src = imgSrc;
   img.border = 0;
   img.hspace = 0;
   img.vspace = 0;
   img.style.verticalAlign = "middle";
   if (tip && tip != null) {
       img.title = tip;
       img.alt = tip;
   }
   return img;
}

/**
 * Creates the action menu link
 */
TableComponent.prototype.createActionMenuLink = function() {
    return this.createLinkIcon("images/showactions.gif",
           bindFunc(this, this.doShowActionMenu, this.entryName),
           this.I18N_ACTION_MENU_HINT,
           this.entryName);
}

/**
 * Creates the title of the table in the specified container
 */
TableComponent.prototype.createTitle = function(title, actions, container) {
   
   var titleTable = document.createElement("table");
   titleTable.cellSpacing = 0;
   titleTable.cellPadding = 0;
   titleTable.width = "100%";
   titleTable.className = "tableheader";
   titleTable.style.borderWidth = "0 0 1px 0";
   
   var titleBody = document.createElement("tbody");
   titleTable.appendChild(titleBody);
   
   var titleRow = document.createElement("tr");
   titleBody.appendChild(titleRow);
   
   var titleText = document.createElement("td");
   titleText.className = "sectionheadertext padding";
   titleText.style.borderWidth = 0;
   titleText.style.textWrap = "none";
   
   if (this.valid == false){
      var errorImg = this.createImage("images/MessageError_16.gif");
      titleText.appendChild(errorImg);
   }
   
//   <span id="ISSUESHORTDESC_reqInd" class="reqInd">*</span>
   var reqID = (this.fieldName) ? this.fieldName + "_": this.entryName;
   var requiredSpan = document.createElement("span");
   requiredSpan.id = reqID.toUpperCase() + "reqInd";
   requiredSpan.className = "reqInd";
   if (this.required != true) {
      requiredSpan.style.display = "none";
   }
   requiredSpan.appendChild(document.createTextNode("*"));
   titleText.appendChild(requiredSpan);

   var indicatorPaddingSpan = document.createElement("span");
   indicatorPaddingSpan.id = reqID.toUpperCase() + "indPad";
   if (this.required != true) {
      indicatorPaddingSpan.style.display = "none";
   }
   indicatorPaddingSpan.appendChild(document.createTextNode(" "));
   titleText.appendChild(indicatorPaddingSpan);
   
   if ((this.valid == false) || (this.required == true)) {
      titleText.appendChild(document.createTextNode(" "));
   }    

   titleText.appendChild(document.createTextNode(title + " ")); 
   //do not disply the action link for print pages   
   if (!isPrintPage() && !this.disableTableActions) {
      titleText.appendChild(this.createActionMenuLink());
   }
   
   var titleHeadTableCell = document.createElement("td");
   var titleHeadTable = document.createElement("table");
   var titleHeadTableBody = document.createElement("tbody");
   var titleHead = document.createElement("tr");
   
   titleHead.appendChild(titleText);
   
   var tableFilterFields = this.tableParameters.getTableFilterFields();
   var tableFilterLabels = this.tableParameters.getTableFilterLabels();
   if (tableFilterLabels != ""){
      for (var i = 0; i < tableFilterLabels.length; i++){
         titleHead.appendChild(this.createFilterDropDown(tableFilterFields[i], tableFilterLabels[i]));
      }
   }
   
   titleHeadTableBody.appendChild(titleHead);
   titleHeadTable.appendChild(titleHeadTableBody);
   titleHeadTableCell.appendChild(titleHeadTable);
   titleRow.appendChild(titleHeadTableCell);
   
   var titleButtons = document.createElement("td");
   titleButtons.className = "tableheaderbuttons";
   titleButtons.style.borderWidth = 0;
   
   if (actions && actions != null) {
      titleButtons.innerHTML = actions;
   }   
   titleRow.appendChild(titleButtons);
   
   container.appendChild(titleTable);   
}

TableComponent.prototype.createFilterDropDown = function(fieldName, label){
   var dropDown = document.createElement("td");
   var availableValues = this.tableParameters.getTableFilterAvailableValues();

   if (availableValues != null){
      var values = availableValues[fieldName];
      
      if (values != null && values.length > 0){
         dropDown.style.paddingLeft = "10px";
         dropDown.appendChild(document.createTextNode(label + ": "));
         
         var select = document.createElement("select");
         var entryName = this.tableParameters.getEntryName();
         var selectFieldName = "select_" + entryName + fieldName;
         select.setAttribute("id", selectFieldName);
         select.onchange = function() {
            getTableComponent(entryName).filterValueChanged(this, fieldName);
         };
         
         var option1 = document.createElement("option");
         option1.setAttribute("value", "_all_");
         option1.appendChild(document.createTextNode(this.FILTER_ALL_OPTION));
         select.appendChild(option1);
         
         for (var i = 0; i < values.length; i++){
            option1 = document.createElement("option");
            option1.appendChild(document.createTextNode(values[i].label));
            option1.setAttribute("value", values[i].value);
            
            if (this.queryFilter.getFilters() != null && this.queryFilter.getFilters().length > 0){
               for (var q = 0; q < this.queryFilter.getFilters().length; q++){
                  var filter = this.queryFilter.getFilter(q);
                  var wrappedValue = "(" + values[i].value + ")";
                  if ((fieldName == filter.columnName) && ( wrappedValue == filter.value) && (filter.operator == '=')){
                     option1.setAttribute("selected", "true");
                  }
               }
            }
            select.appendChild(option1);
         }
         dropDown.appendChild(select);
      }
   }
   return dropDown;
}

TableComponent.prototype.filterValueChanged = function(select, fieldName){
   //var select = document.getElementById(selectFieldName);
   var optionValue = "(" + select.options[select.selectedIndex].value + ")";
   var optionLabel = "(" + select.options[select.selectedIndex].text + ")";
  
   var qf = new QueryFilter(this.tableParameters);
   var key = qf.getFilterKeyByColumnName(fieldName);
   if (optionValue == "(_all_)" && key != -1){
      qf.removeFilter(key);
   }
   else {
      if (key != -1){
         qf.updateFilter(key, fieldName, '=', optionValue, optionLabel)
      }
      else {
         qf.addFilter(fieldName, '=', optionValue, optionLabel)
      }
   }
   this.tableParameters.setQueryFilters(qf.getFilters());
   this.tableParameters.setStart(0);
   this.tableParameters.resetCurrentView();
   this.submit();
}

/**
 * Creates the row of dialog buttons below the table
 */
TableComponent.prototype.createDialogButtons = function() {
   var table = document.createElement("table");
   //table.id = "dialogButtons";
   table.width = "100%";
   table.cellSpacing = "0";
   table.cellPadding = "0";
   tbody = document.createElement("tbody");
   table.appendChild(tbody);
   
   var tr = document.createElement("tr");
   tbody.appendChild(tr);
   
   var td = document.createElement("td");
   td.style.paddingTop = "8px";
   tr.appendChild(td);
   
   var button = this.createButtonCell
      ("done", this.I18N_BUTTON_DONE, 100, this.doDone);
   td.appendChild(button);

   return table;
}

/**
 * Utility routine used by createButtonRow
 */
TableComponent.prototype.createButtonCell = function(name, value, width, onclick) {
   var button = document.createElement("input");
   button.width = width;
   button.name = name;
   button.type = "button";
   button.value = value;
   button.onclick = onclick;
   
   // style
   button.className = "button";

   return button;
}

/**
 * Returns the number of rows
 */
TableComponent.prototype.getRowCount = function() {

   // Use cached value if this has already been calculated

   if (this.rowCount == null) {
      var tableParameters = this.tableParameters;
      var n = 0;
      for (var row = 0;; row++) {
         if (!tableParameters.cellExists(row, 0)) {
            break;
         }
         n++;
      }
      this.rowCount = n;
   }
   return this.rowCount;
}

/**
 * Copy the current table parameters from the query form 
 * into a temporary form that will bu used to submit 
 * to the ExportToXLS servlet.
 */
TableComponent.prototype.buildExportForm = function(queryForm,enrich,includeDims,includeAllCustoms) {

   var tc = getTableComponent(this.entryName);
   var exportXlsUrl = tc.exportXlsUrl;
   var exportFormName = "ExportForm";

   // Create the form to be used for download
   var result = document.getElementById(exportFormName);

   if (result == null)
      result = document.createElement("form");
   else
      result.innerHTML = ""; // removes all child nodes
   
   result.id = exportFormName;
   result.name = exportFormName;
   
   if (exportXlsUrl && exportXlsUrl != '')
     result.action = tc.createExportAction(exportXlsUrl,enrich);
   else 
     result.action = tc.createExportAction(queryForm.action,enrich);
     
   result.method = "post";

   for (var i = 0; i < queryForm.elements.length; i++) {

      // Only interested in <input type="HIDDEN"> elements

      var element = queryForm.elements[i];
      if (element.tagName.toUpperCase() != "INPUT") continue;

      var elementType = element.getAttribute("type");
      if (elementType.toUpperCase() != "HIDDEN") continue;

      // Ignore cells and handles - download doesn't use them

      var elementName = element.getAttribute("name");
      var elementValue = element.getAttribute("value");

      if (elementName.match(/^TCCell/)) continue;
      if (elementName.match(/^TCHandle/)) continue;
      if (elementName.match(/^TCAllFieldNames/)) continue;

      // Set the start and count parameters to download the
      // entire query

      if (elementName == this.entryName + "TCStart")
         elementValue = "0";
      if (elementName == this.entryName + "TCCount")
         elementValue = tc.MAX_DOWNLOADABLE_ROWS;
         
      // Create one parameter line in the new form   

      var param = document.createElement("input");
      param.setAttribute("type", "hidden");
      param.setAttribute("name", elementName);
      param.setAttribute("value", elementValue);

      result.appendChild(param);
   }
   // Enrichment flags
   var enrichParam = document.createElement("input");
   enrichParam.setAttribute("type", "hidden");
   enrichParam.setAttribute("name", "TCEnrichment");
   enrichParam.setAttribute("value", enrich?"true":"false");
   result.appendChild(enrichParam);
   var enrichDimsParam = document.createElement("input");
   enrichDimsParam.setAttribute("type", "hidden");
   enrichDimsParam.setAttribute("name", "TCEnrichIncludeDims");
   enrichDimsParam.setAttribute("value", includeDims?"true":"false");
   result.appendChild(enrichDimsParam);
   var enrichAllCustomsParam = document.createElement("input");
   enrichAllCustomsParam.setAttribute("type", "hidden");
   enrichAllCustomsParam.setAttribute("name", "TCEnrichIncludeAllCustoms");
   enrichAllCustomsParam.setAttribute("value", includeAllCustoms?"true":"false");
   result.appendChild(enrichAllCustomsParam);


   return result;
}

/**
 * Create the action property to be used by the form
 */
TableComponent.prototype.createExportAction = function(queryAction,enrich) {
   // Find the current end of the parameter list

   var p = queryAction.lastIndexOf('?');
   if (p == -1) {
      queryAction += "?";
   }
   else {
      queryAction += "&";
   }
   queryAction += "tcmethod=export";
   if(enrich) {
      queryAction += "&tcenrich=true";
   }
   
   if (queryAction.indexOf('|') > 0){
      queryAction = encodeURI(queryAction);
   }
   return queryAction;
}


/**
 * Create the action property to be used by the Customization information display table
 */
TableComponent.prototype.createShowCustomizationAction = function(queryAction) {
    if (queryAction.indexOf('|') > 0){
       queryAction = encodeURI(queryAction);
    }
   
	var queryIndex = queryAction.indexOf("?");
	if (queryIndex != -1) {
		var queryPath = queryAction.substring(0, queryIndex);
		var queryString = queryAction.substring(queryIndex+1);
		
		var oldParams = queryString.split("&");
		
		var newParams = ["tcmethod=showCustomizationInfo"];
		for (var i=0; i < oldParams.length; i++) {
			// Copy non-tcmethod params
			var oldParam = oldParams[i];
			if (oldParam && oldParam.indexOf("tcmethod=") != 0) {
				newParams.push(oldParam);
			}
		}
		return queryPath + "?" + newParams.join("&");
	}
	else {
		return queryAction + "?tcmethod=showCustomizationInfo";
	}
}

// ==============================================
//    Event handlers.  Note that "this" in the
//    context of an event handler refers to
//    the HTML element containing the "onClick"
//    that invokes the event handler.  In this
//    case, it's the <input type="button">
//    element.
//
//    In order to allow the event handler to
//    access methods in the TableComponent
//    object, we store a reference to it in the
//    document object as "document.tableComponent"
// ==============================================

/**
 * Handles the "first" button click event
 */
TableComponent.prototype.doFirst = function() {
   var tc = getTableComponent(this.entryName);
   var tableParameters = tc.tableParameters;
   var start = 0;
   tableParameters.setStart(start);
   tc.submit();
}

/**
 * Handles the "prev" button click event
 */
TableComponent.prototype.doPrev = function() {
   var tc = getTableComponent(this.entryName);
   var tableParameters = tc.tableParameters;
   var count = tableParameters.getCount();
   var start = tableParameters.getStart();
   start -= count;
   if (start < 0)
      start = 0;
   tableParameters.setStart(start);
   tc.submit();
}

/**
 * Handles the "done" button click event
 */
TableComponent.prototype.doDone = function() {
    // the Done button expects the table component query actions to drop a breadcrumb
    // to itself; therefore, doDone will jump two spots back.
    var s = window.name.toLowerCase();

    if (window.name.indexOf("popup") == -1)
       window.location.href = "Breadcrumb.do?n=2";
    else
       window.close();
}

/**
 * Handles the "customize" button click event
 */
TableComponent.prototype.doCustomize = function() {
   var tc = getTableComponent(this.entryName);
   var win = IFrameDialog.show({
      url: "TableCustomizer.do?entryId=" + this.entryName,
      title: AppProperties["tableCustomizer.title"],
      name: "_blank",
      width: 750,
      height: 300});
}


/**
 * Handles the "customize" button click event
 */
TableComponent.prototype.doShowCustomizationData = function() {
   var tc = getTableComponent(this.entryName);
   var tableParameters = tc.tableParameters;
   var form = tableParameters.getForm();
   var submitURL;
   if (tc.exportXlsUrl){
	   submitURL= tc.createShowCustomizationAction(tc.exportXlsUrl);
   } else{
	   submitURL = tc.createShowCustomizationAction(form.action);
   } 
   var win = IFrameDialog.show({
      url: submitURL,
      title: tc.I18N_BUTTON_CUSTOMIZE_INFO,
      name: "_blank",
      width: 900,
      height:600});
}




/**
 * Handles the "query" button click event
 */
TableComponent.prototype.doQuery = function() {
   var win = IFrameDialog.show({
      url: this.getQueryEditorURL(),
      title: AppProperties["queryEditor.title"],
      name: "_blank",
      width: 700,
      height: 425});
}

TableComponent.prototype.getQueryEditorURL = function()
{
   return "QueryEditor.do?entryId=" + this.entryName;
}

/**
 * Handles the "Export" button click event
 */
TableComponent.prototype.doExport = function(enrich,includeDims,includeAllCustoms) {

   // Get this document's form
   var tc = getTableComponent(this.entryName);  
   var tableParameters = tc.tableParameters;
   var form = tableParameters.getForm();
   
   var exportForm = tc.buildExportForm(form,enrich,includeDims,includeAllCustoms);
   if (exportForm == null) return;
   
   var prevForm = document.getElementById(exportForm.id);
   
   if (prevForm == null) {
      document.body.appendChild(exportForm);
   }
   
   exportForm.submit();
}

/**
 * Shows the Enrichment Options
 */
TableComponent.prototype.pickEnrichmentOptions = function() {
   var win = IFrameDialog.show({
      url: "EnrichmentOptions.do",
      title: this.I18N_ENRICH_OPTIONS_TITLE,
      name: "_blank",
      id: "enrichmentOptions"});
}
/**
 * Handles the "next" button click event
 */
TableComponent.prototype.doNext = function() {
   var tc = getTableComponent(this.entryName);
   var tableParameters = tc.tableParameters;
   var count = tableParameters.getCount();
   var start = tableParameters.getStart();
   var total = tableParameters.getTotalRows();
   if (total == 0 || (total > start + count)) {
      tableParameters.setStart(start + count);
      tc.submit();
   }
}

/**
 * Handles the "last" button click event
 */
TableComponent.prototype.doLast = function() {
   var tc = getTableComponent(this.entryName);
   var tableParameters = tc.tableParameters;
   var count = tableParameters.getCount();
   var start = tableParameters.getStart();
   var total = tableParameters.getTotalRows();
   var pages = Math.ceil(total / count);
   tableParameters.setStart((pages - 1) * count);
   tc.submit();
}

/**
 * Handles all submit actions, both from this class
 * and the customizer/query windows.
 */
TableComponent.prototype.submit = function() {
   var tc = getTableComponent(this.entryName);
   var tableParameters = tc.tableParameters;
   var form = tableParameters.getForm();
   
   if (this.wizCursor && this.wizCursor != '' && form.action && form.action.indexOf('?')===-1) form.action = form.action + "?method=" + this.wizCursor;
   
 //  alert("Action: " + form.action);
   form.submit();
}

/**
 * Handles a column sort request
 */
TableComponent.prototype.doOrderBy = function(name) {
   var tc = getTableComponent(this.entryName);
   var tableParameters = tc.tableParameters;
   var orderByName, orderByDirection;

   var orderByList = tableParameters.getOrderBy();
   if (orderByList == null) {
      orderByName = name;
      orderByDirection = 0;
   }
   else {
      orderByName = orderByList[0];
      orderByDirection = (orderByName == name) 
         ? orderByDirection = 1 - orderByList[1] : 0;
      orderByName = name;
   }
   
   tableParameters.setOrderBy(orderByName, orderByDirection);
   tableParameters.resetCurrentView();
   tableParameters.setStart(0);
   tc.submit();
}

/**
 * Handles the "action menu" click event
 */
TableComponent.prototype.doShowActionMenu = function(entry_name) {
   var tc = getTableComponent(entry_name);
   var menu = new TableMenu();
   
   if (!entry_name)
   {
      entry_name = '';
   }
   
   menu.addMenuItem(new MenuItem(MenuItem.prototype.action, "images/edit.gif", tc.I18N_BUTTON_CUSTOMIZE, "javascript:getTableComponent('" + entry_name + "').doCustomize()"));
   
   var tableParameters = tc.tableParameters;
   var superUser = tableParameters.getSuperUser();
   var superUser = (superUser == 1);
   if (superUser){
	   menu.addMenuItem(new MenuItem(MenuItem.prototype.action, "images/edit.gif", tc.I18N_BUTTON_CUSTOMIZE_INFO, "javascript:getTableComponent('" + entry_name + "').doShowCustomizationData()"));
   }
   menu.addMenuItem(new MenuItem(MenuItem.prototype.action, "images/search.gif", tc.I18N_BUTTON_QUERY, "javascript:getTableComponent('" + entry_name + "').doQuery()"));
   menu.addMenuItem(new MenuItem(MenuItem.prototype.action, "images/ExportTable.gif", tc.I18N_BUTTON_EXPORT, "javascript:getTableComponent('" + entry_name + "').doExport()"));
   
   menu.render();
   return false;
}

TableComponent.prototype.getParameters = function() {
    return this.tableParameters;
}

function getTableComponentIndex (entry_name){
   var tableComponent = getTableComponent(entry_name);
   return (tableComponent) ? tableComponent.index : -1;
}

function FixedColumn(index, width, cssClass) {
   this.index = index;
   this.width = width;
   this.cssClass = cssClass;
}

function bindFunc(obj, func, entry_name){
   return function() {
      return func.apply(obj, [ entry_name ]);;
   }
}

function isPrintPage(){
   var printPage =false;
      if (document.getElementById("printPage")  != null &&
            document.getElementById("printPage").value == "1") {
         printPage = true;
      }
      return printPage;
}

function getParamsForTable(form, entryName, fieldName) {
   var tcPrefix = "TC";
   if (entryName) {
      var tc = TableComponent.getTableComponent(entryName, fieldName);
      tcPrefix = tc.entryName + "TC";
   }
   var cpbPrefix = "cpb_";    // won't hurt to also include a few CPB params
   return getFieldMap(form, 
      function(element) {
         return (element.type == 'hidden' && 
            ((element.name.indexOf(tcPrefix) == 0) ||
             (element.name.indexOf(cpbPrefix) == 0)));
      });
}

function getParamsExcludeTable(form) {
   return getFieldMap(form, 
      function(element) {
         return isInterestingFieldElement(element) &&
            !(element.type == 'hidden' && (element.name.indexOf("_TC") >= 0));
      });
}
