/////////////////////////////////////////
//  Catch-all fix for pipe characters in requests
//
//    In SAS 9.4M5, a tighter security in the AppServer is introduced, to immediately return 400 error 
//    when there are illegal characters in the request URL, which includes the pipe character "|" that 
//    we used all over the places as handle separators. There are a lot of places changed to apply 
//    "encodeURI()" to encode the pipe character in the request are encoded so it won't block by the 
//    AppServer. 
//
//    This is a backup-insurance to try to catch all other places that we could not manually identified. 
//    For all links, form actions and window.open URL, it will replace all "|" to "%7C"
/////////////////////////////////////////

for(var i=0;i<document.forms.length;i++){
   var f=document.forms[i];
   var func=function(origOnSubmit,origSubmit){
      f.onsubmit=function(){
         //console.error("GOTCHA FORM");
         if(origOnSubmit){
            return origOnSubmit.apply(this,arguments);
         }
      };
      f.submit=function(){
         //console.error("GOTCHA FORM SUBMIT");
         
         // Fix for pipe characters in form submits
         if (this.action && this.action.replace) {
            console.error("Post-processing Action: " + this.action);
            this.action = this.action.replace(/\|/g, "%7C");
         }
         
         /////////////////////////////////////////
         // Fix for XSS Auditor error
         //
         //   Also in SAS 9.4M5, an increase in security by enabling X-XSS-Protection to "1; mode=block" in 
         //   the platform caused some pages are blocked by Chrome, with error 
         //   "The XSS Auditor refused to execute a script in 'https://XXXXXXXXXXXX' because its source code was found within the request."
         //
         //   Later it is found when we build the Query Tables in old UI, the hidden inputs that helps TableComponent 
         //   to build select actions and fixed columns may contains some Javascript, and sends together in the form 
         //   during submit. It was fine in the past, but with X-XSS-Protection enabled, the Javascript in these 
         //   hidden inputs in the submitted form in requests may confuse the browser that these inputs which contains 
         //   Javascript definitions, may try to hijack the definitions of other Javascript definitions in the response page, 
         //   and block rendering the response page as requested.
         //
         //   As a result, the solution is to eliminate the values of these hidden input fields. In our Action classes, 
         //   these values are not being used anyway. 
         //
         //   For more information about "X-XSS-Protection", 
         //   see https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-XSS-Protection
         /////////////////////////////////////////
         var filterNames = ["TCFixCol", "TCActionsHTML", "TCSelectActionName"];
         var formInputs = this.getElementsByTagName("input");
         if (formInputs && formInputs.length){
            for (var j = 0 ; j < formInputs.length; j++){
               for (var p = 0; p < filterNames.length; p++){
                  if (formInputs[j].name.indexOf(filterNames[p]) >= 0 && formInputs[j].value ) {
                     formInputs[j].value = '';
                  }
               }
            }
         }
         // End of fix for XSS Auditor
         
         if(origSubmit){
            return origSubmit.apply(this,arguments);
         }
      };
   };
   func(f.onsubmit,f.submit);
}

// Fix for links URL
for(var i=0;i<document.links.length;i++){
   var l=document.links[i];
   var func=function(origOnClick){
      l.onclick=function(){
         //console.error("GOTCHA LINK");
         if (this.href && this.href.indexOf("http") == 0) {
            console.error("Post-processing HREF: " + this.href);
            this.href = this.href.replace(/\|/g, "%7C");
         }
         if(origOnClick){
            return origOnClick.apply(this,arguments);
         }
      };
   };
   func(l.onclick);
}

// Fix for window.open's URL
(function(origWindowOpen){
   window.open=function(){
      //console.error("WRAPPED WINDOW OPEN");
      if (arguments && arguments.length){
         console.error("Post-processing URL: " + arguments[0]);
         arguments[0] = arguments[0].replace(/\|/g, "%7C");
      }
      return origWindowOpen.apply(this,arguments);
   }; 
})(window.open);
