/**********************************************************************/
/*  Copyright (c) 2005-2014 by SAS Institute, Inc., Cary, NC, USA.    */
/*  All rights reserved.                                              */
/**********************************************************************/
/*  NAME:        orm_db_connect.sas                                   */
/*  VERSION:     6.1                                                  */
/*  DESCRIPTION: Shared macro to build the SAS connect statement for  */
/*               explicit pass-through to an RDBMS.                   */
/*  PARAMETERS:  ConnectName = DB connection alias name               */
/*                 (default: SASOPRSK_CONN)                           */
/*  PRODUCT:     SAS Enterprise GRC                                   */
/**********************************************************************/

%macro orm_db_connect(ConnectName=SASOPRSK_CONN);
  
%put NOTE: {START: &sysmacroname};
  
  %global DBConnectSQL DBConnectAlias;

  %let DBConnectAlias = &ConnectName;

  /*  For ORACLE: Build the DBConnectSQL macro variable */
  %if %upcase(&orm_dbserver) = ORACLE %then 
  %do;
    %let DBConnectSQL = connect to &orm_dbengine as &DBConnectAlias 
       (path="&orm_dbName" authdomain="&orm_dbAuth" connection=global dbmax_text=4000);
  %end;

  /*  For DB2: Build the DBConnectSQL macro variable */
  %if %upcase(&orm_dbserver) = DB2 %then 
  %do;
    %let DBConnectSQL = connect to &orm_dbengine as &DBConnectAlias 
       (database="&orm_dbName" authdomain="&orm_dbAuth"  connection=global autocommit=no dbmax_text=4000);
  %end;

  /*  For PostgreSQL: Build the DBConnectSQL macro variable */
  %if %upcase(&orm_dbserver) = POSTGRESQL %then
  %do;
    %let DBConnectSQL = connect to &orm_dbengine as &DBConnectAlias 
       (server="&orm_dbServerNm" port="&orm_dbport" database="&orm_dbName" authdomain="&orm_dbAuth" connection=global autocommit=no dbmax_text=4000);
  %end;
  
  /*  For SQLServer: Build the DBConnectSQL macro variable */
  %if %upcase(&orm_dbserver) = SQLSERVER %then 
  %do;    
    %let DBConnectSQL = connect to &orm_dbengine as &DBConnectAlias 
       (dsn="&orm_dbName" authdomain="&orm_dbAuth"  connection=global autocommit=no dbmax_text=1000);
  %end;
  
%put NOTE: {END: &sysmacroname};

%mend orm_db_connect;
