/************************************************************************************/
/*  Copyright (c) 2005-2014 by SAS Institute, Inc., Cary, NC, USA.                  */
/*  All rights reserved.                                                            */
/************************************************************************************/
/*  NAME:        orm_get_cust_field_labels.sas                                      */
/*  VERSION:     6.1                                                                */
/*  DESCRIPTION: Build a table (customMessages) of default and custom field labels. */
/*  PARAMETERS:                                                                     */
/*  PRODUCT:    SAS Enterprise GRC                                                  */
/************************************************************************************/
%macro orm_get_cust_field_labels;

%put NOTE: {START: &sysmacroname};

  /* Get the custom field label from the sashelp.ormonrpt table */
  %if %sysfunc(exist(sashelp.ormonrpt)) %then
  %do;
    data ormonrpt(drop=lineno);
      length locale $5 key $200 text $1800;
      set sashelp.ormonrpt;
      where upcase(locale)=upcase("&localecd") or
            upcase(locale)="%substr(%upcase(&localecd),1,2)";
      locale=%upcase("&localecd");
    run;
  %end;

  /* Construct the content server path for the customMessages.txt file to get the custom field labels */
  
  /* Get the content server URL */
  %if not %symexist(service_endpoint) %then %global service_endpoint;
  %svcreg(type=REST, name=ServerService REST);
  %let grc_dav_srv=&service_endpoint;
  
  /* Parse the URL to keep just the server and port information */  
  %let grc_srv_url=%substr(&grc_dav_srv,1,(%sysfunc(indexw("&grc_dav_srv",%qscan(%bquote(&grc_dav_srv),3,'/'),"/"))-3));

  /* Get the remainder of the DAV path from SMC */
  %global grc_dav_path;
  
  %orm_get_app_property_value(application=&orm_application_name, property=Dav.url.path,
                              output_property_value=orm_dav_path, output_status=orm_status);

  %let grc_dav_path=%trim(%nrstr(&orm_dav_path));
  
  /* Put the content server URL and DAV path together */
  %let grc_dav_url=&grc_srv_url.&grc_dav_path./Config;
      
  /* first look for full locale (e.g. "en_US") customMessages_xx_xx.properties file */
  %if %length(&localecd) > 2 %then
  %do;
    filename svc_rsp temp;
    
    proc http url="&grc_dav_url./customMessages_%substr(%lowcase(&localecd),1,2)_%substr(%upcase(&localecd),4).properties"
              out=svc_rsp
              method="get"
              ct="application/x-www-form-urlencoded" 
              HTTP_TOKENAUTH
    ;
    run;  
     
    data customloc;
      infile svc_rsp recfm=v;
      input x $char2000. @@;
    run;
      
    filename svc_rsp;

   %local nobs rc;
    %let dsid = %sysfunc(open(work.customloc));
    %if &dsid %then 
    %do;
      %let nobs  = %sysfunc(attrn(&dsid, nlobs));
      %let rc = %sysfunc(close(&dsid));
    %end;
    %else %let nobs  = 0;
 
    %if &nobs > 0 %then 
    %do;
     data customloc;
       set customloc;
       x=unicode(x,'esc');
     run;

     data customloc(drop=x);
       set customloc(where=(x contains "="));
       length locale $5 key $200 text $1800 ;
       locale="%upcase(&localecd)";
       key=prxchange('s/(^.*)\.field\.(.*)\.displayName\.txt\s*$/$1_OBJ_$2/i',-1,scan(upcase(x),1,"="));
       text=scan(x,-1,"=");
      if substr(key,1,1)='#' then delete;
     run;

     /* Sort the default labels */
     proc sort data=ormonrpt;
       by key;
     run;

     /* Now merge in the locale specific labels */
     proc sort data=customloc;
      by key;
     run;

     data Custommessages;
       merge ormonrpt(in=a) customloc(in=b);
       by key;
     run;
	 
	 proc sort data=Custommessages nodupkey;
       by key;
     run;
   %end;
  %end;
  /* second look for short locale (e.g. "en") customMessages_xx.properties file */
  %else %if %length(&localecd) = 2 %then
  %do;
    filename svc_rsp temp;
    
    proc http url="&grc_dav_url./customMessages_%substr(%lowcase(&localecd),1).properties"
              out=svc_rsp
              method="get"
              ct="application/x-www-form-urlencoded" 
              HTTP_TOKENAUTH
    ;
    run;  
     
    data customloc;
      infile svc_rsp recfm=v;
      input x $char2000. @@;
    run;
      
    filename svc_rsp;
    
    %local nobs rc;
    %let dsid = %sysfunc(open(work.customloc));
    %if &dsid %then 
    %do;
      %let nobs  = %sysfunc(attrn(&dsid, nlobs));
      %let rc = %sysfunc(close(&dsid));
    %end;
    %else %let nobs  = 0;
 
    %if &nobs > 0 %then 
    %do; 
      data customloc;
        set customloc;
        x=unicode(x,'esc');
      run;

      data customloc(drop=x);
        set customloc(where=(x contains "="));
        length locale $5 key $200 text $1800 ;
        locale="%substr(%upcase(&localecd),1,2)";
        key=prxchange('s/(^.*)\.field\.(.*)\.displayName\.txt\s*$/$1_OBJ_$2/i',-1,scan(upcase(x),1,"="));
        text=scan(x,-1,"=");
      if substr(key,1,1)='#' then delete;
      run;

      /* Sort the default labels */
      proc sort data=ormonrpt;
        by key;
      run;

      /* Now merge in the locale specific labels */
      proc sort data=customloc;
        by key;
      run;

      data Custommessages;
        merge ormonrpt customloc;
        by key;
      run;
	  
	  proc sort data=Custommessages nodupkey;
       by key;
      run;
    %end;  
  %end;

  /* Read labels from customMessages.properties file if there is no locale specific version of this file */
  %local nobs rc;
  %let dsid = %sysfunc(open(work.customloc));
  %if &dsid %then 
  %do;
    %let nobs  = %sysfunc(attrn(&dsid, nlobs));
    %let rc = %sysfunc(close(&dsid));
  %end;
  %else %let nobs  = 0;
 
  %if &nobs = 0 %then 
  %do;
    filename svc_rsp temp;
    
    proc http url="&grc_dav_url./customMessages.properties"
              out=svc_rsp
              method="get"
              ct="application/x-www-form-urlencoded" 
              HTTP_TOKENAUTH
    ;
    run;  
     
    data customen;
      infile svc_rsp recfm=v;
      input x $char2000. @@;
    run;
      
    filename svc_rsp;
      
    data customen(drop=x);
      set customen(where=(x contains "="));
      length locale $5 key $200 text $1800 ;
      locale="EN";
      key=prxchange('s/(^.*)\.field\.(.*)\.displayName\.txt\s*$/$1_OBJ_$2/i',-1,scan(upcase(x),1,"="));
      text=scan(x,-1,"=");
      if substr(key,1,1)='#' then delete;
    run;

    /* Sort the default labels */
    proc sort data=ormonrpt;
      by key;
    run;

    /* Now merge in the locale specific labels */
    proc sort data=customen;
      by key;
    run;

    data Custommessages;
      merge ormonrpt customen;
      by key;
    run;
	
	proc sort data=Custommessages nodupkey;
      by key;
    run;
  %end;
  
%put NOTE: {END: &sysmacroname};

%mend orm_get_cust_field_labels;
